/*
  RCTractor.h - Library for controlling RC Tractors.
  Created by Oisin O'Conchubhair, September 20, 2015.
  Released into the public domain.

  This library uses parts from pcf8574.cpp written by Rob Tillaart.
*/

#include "Arduino.h"
#include "RCTractorGuy.h"
#include <RCTractorGuyI2C.h>

#include <Servo.h>

// Pin Assignments
static int _Steering_Servo_Pin = 0;
static int _Rear_Link_Servo_Pin = 0;
static int _Front_Link_Servo_Pin = 0;
static int _Motor_1_PWM_Pin = 0;
static int _Motor_1_Direction_1_Pin = 0;
static int _Motor_1_Direction_2_Pin = 0;
static int _Motor_2_PWM_Pin = 0;
static int _Motor_2_Direction_1_Pin = 0;
static int _Motor_2_Direction_2_Pin = 0;
static int _Head_Lights_Pin = 0;
static int _Tail_Lights_Pin = 0;
static int _Work_Lights_Pin = 0;
static int _Accessory_Lights_Pin = 0;
static int _Left_Indicator_Pin = 0;
static int _Right_Indicator_Pin = 0;
static int _Left_Beacon_Pin = 0;
static int _Right_Beacon_Pin = 0;

#define SOP '<'
#define EOP '>'

static int _Diff_Adj_1;
static int _Diff_Adj_2;
static int _PCF_Data;
static int _Motor_con2;
static int _Motor_1_PWM_Value;
static int _Motor_2_PWM_Value;
static int _Left_Ind;
static int _Right_Ind;
static int _Left_Beacon;
static int _Right_Beacon;
static int _Left_Ind_On;
static int _Right_Ind_On;
static int _Timer1_Counter;
static int _Light_Val;
static int _Old_Light_Val;
static int _Head_Lights;
static int _Brake_Lights;
static int _Spot_Lights;
static int _Feature_Lights;
static int _Flash_Timer;
static int _Flash_Timer2;
static int _Left_Ind_Status;
static int _Right_Ind_Status;
static int _Beacon_Option = 2;
static int _HeadLight_Option = 0;
static int _PCF8574 = 1;
static int _Drive_Val = 0;

// Motor Configuration Variables

static int _Motor_Config = 1;
static int _Use_PCF8574 = 0;
static int I2C_Add = 0x38;
static int _Steering_Config = 1;
static int _Use_PCF8574_Steer = 0;

// Servo Variables
static int _Steering_Servo_Min = 0x00;
static int _Steering_Servo_Max = 0xFF;
static int _Rear_Link_Servo_Min = 0x00;
static int _Rear_Link_Servo_Max = 0xFF;
static int _Front_Link_Servo_Min = 0x00;
static int _Front_Link_Servo_Max = 0xFF;
static int _Steering_Val = 0;
static int _Rear_Link_Val = 0;
static int _Front_Link_Val = 0;

/************************************************************/
/*       Functions used to control RC vehicles              */
/************************************************************/


RCTractor::RCTractor(int Motor_1_PWM, int Motor_1_Dir_1, int Motor_1_Dir_2, int Motor_2_PWM, int Motor_2_Dir_1, int Motor_2_Dir_2, int Steering, int Rear_Link, int Front_Link, int Left_Ind,int Right_Ind,int Head_Lights,int Tail_Lights,int Work_Lights,int Accessory_Lights,int Left_Beacon, int Right_Beacon)
{

	_Motor_1_PWM_Pin = Motor_1_PWM;
	_Motor_1_Direction_1_Pin = Motor_1_Dir_1;
	_Motor_1_Direction_2_Pin = Motor_1_Dir_2;
	_Motor_2_PWM_Pin = Motor_2_PWM;
	_Motor_2_Direction_1_Pin = Motor_2_Dir_1;
	_Motor_2_Direction_2_Pin = Motor_2_Dir_2;
	_Steering_Servo_Pin = Steering;
	_Rear_Link_Servo_Pin = Rear_Link;
	_Front_Link_Servo_Pin = Front_Link;
	_Left_Indicator_Pin = Left_Ind;
	_Right_Indicator_Pin = Right_Ind;
	_Head_Lights_Pin = Head_Lights;
	_Tail_Lights_Pin = Tail_Lights;
	_Work_Lights_Pin = Work_Lights;
	_Accessory_Lights_Pin = Accessory_Lights;
	_Left_Beacon_Pin = Left_Beacon;
	_Right_Beacon_Pin = Right_Beacon;


	if(_Motor_1_PWM_Pin!=0){pinMode(_Motor_1_PWM_Pin, OUTPUT);}
	if(_Motor_1_Direction_1_Pin!=0){pinMode(_Motor_1_Direction_1_Pin, OUTPUT);}
	if(_Motor_1_Direction_2_Pin!=0){pinMode(_Motor_1_Direction_2_Pin, OUTPUT);}
	if(_Motor_2_PWM_Pin!=0){pinMode(_Motor_2_PWM_Pin, OUTPUT);}
	if(_Motor_2_Direction_1_Pin!=0){pinMode(_Motor_2_Direction_1_Pin, OUTPUT);}
	if(_Motor_2_Direction_2_Pin!=0){pinMode(_Motor_2_Direction_2_Pin, OUTPUT);}
	if(_Steering_Servo_Pin!=0){pinMode(_Steering_Servo_Pin, OUTPUT);}
	if(_Rear_Link_Servo_Pin!=0){pinMode(_Rear_Link_Servo_Pin, OUTPUT);}
	if(_Front_Link_Servo_Pin!=0){pinMode(_Front_Link_Servo_Pin, OUTPUT);}
	if(_Left_Indicator_Pin!=0){pinMode(_Left_Indicator_Pin, OUTPUT);}
	if(_Right_Indicator_Pin!=0){pinMode(_Right_Indicator_Pin, OUTPUT);}
	if(_Head_Lights_Pin!=0){pinMode(_Head_Lights_Pin, OUTPUT);}
	if(_Tail_Lights_Pin!=0){pinMode(_Tail_Lights_Pin, OUTPUT);}
	if(_Work_Lights_Pin!=0){pinMode(_Work_Lights_Pin, OUTPUT);}
	if(_Accessory_Lights_Pin!=0){pinMode(_Accessory_Lights_Pin, OUTPUT);}
	if(_Left_Beacon_Pin!=0){pinMode(_Left_Beacon_Pin, OUTPUT);}
	if(_Right_Beacon_Pin!=0){pinMode(_Right_Beacon_Pin, OUTPUT);}


	_PCF_Data = B00000000;

	I2c.begin();
	I2c.write(I2C_Add,0x02,_PCF_Data);
	bitSet(_PCF_Data,6);

}

void RCTractor::begin(){
	if(_Steering_Servo_Pin!=0){
		_Steering_Servo.attach(_Steering_Servo_Pin);
	}

	if(_Rear_Link_Servo_Pin!=0){
		_Rear_Link_Servo.attach(_Rear_Link_Servo_Pin);
	}

	if(_Front_Link_Servo_Pin!=0){
		_Front_Link_Servo.attach(_Front_Link_Servo_Pin);
	}

	  // Start Serial Communications
	  Serial.begin(9600);
}

void RCTractor::Trailer(uint8_t buf[16]){
	Serial.write(SOP);
	Serial.write(0x01);
	Serial.write(buf[1]);
	Serial.write(buf[2]);
	Serial.write(buf[3]);
	Serial.write(buf[4]);
	Serial.write(buf[5]);
	Serial.write(buf[6]);
	Serial.write(buf[7]);
	Serial.write(buf[8]);
	Serial.write(buf[9]);
	Serial.write(buf[10]);
	Serial.write(EOP);
}
void RCTractor::SetServoLimits(int Ser1_Min, int Ser1_Max, int Ser2_Min, int Ser2_Max, int Ser3_Min, int Ser3_Max){
_Steering_Servo_Min = Ser1_Min;
_Steering_Servo_Max = Ser1_Max;
_Rear_Link_Servo_Min = Ser2_Min;
_Rear_Link_Servo_Max = Ser2_Max;
_Front_Link_Servo_Min = Ser3_Min;
_Front_Link_Servo_Max = Ser3_Max;
}

void RCTractor::MotorConfiguration(int MotorConfig, int UsePCF){
	_Motor_Config = MotorConfig;
	_Use_PCF8574 = UsePCF;
}

void RCTractor::SteeringConfiguration(int SteeringConfig, int UsePCFSteer){
	_Steering_Config = SteeringConfig;
	_Use_PCF8574_Steer = UsePCFSteer;
}

void RCTractor::Drive(int New_Drive_Val,int New_Steering_Val){

	if(_Steering_Config == 1){
	_Steering_Val = New_Steering_Val;
	int _Steering_Val_Map = map(New_Steering_Val, 0x00, 0xFF, _Steering_Servo_Min, _Steering_Servo_Max); // Scale data_val to between steering_min and steering_max
	_Steering_Servo.write(_Steering_Val_Map);
	}
	else if(_Steering_Config == 2){

		if(_Use_PCF8574_Steer==0){
					if (New_Steering_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
			    		_Motor_2_PWM_Value = map(New_Steering_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
	    				digitalWrite(_Motor_2_Direction_1_Pin, HIGH); // Set the direction with pins 7 and 8
	    				digitalWrite(_Motor_2_Direction_2_Pin, LOW);
					}
	   				else if (New_Steering_Val > 135){
	    				_Motor_2_PWM_Value = map(New_Steering_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
	   	 				digitalWrite(_Motor_2_Direction_2_Pin, HIGH); // Set the direction with pins 7 and 8
	    				digitalWrite(_Motor_2_Direction_1_Pin, LOW);
	 				}
	   				else{
	   	 				_Motor_2_PWM_Value=0;
	    				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
	    				digitalWrite(_Motor_2_Direction_1_Pin, LOW); // Set the direction with pins 7 and 8
	    				digitalWrite(_Motor_2_Direction_2_Pin, LOW);
	   				}
				}

				if(_Use_PCF8574_Steer==1){
					if (New_Steering_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
			    		_Motor_2_PWM_Value = map(New_Steering_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

						bitSet(_PCF_Data,0);
						bitClear(_PCF_Data,7);
						I2c.write(I2C_Add,0x02,_PCF_Data);

					}
	   				else if (New_Steering_Val > 135){
	    				_Motor_2_PWM_Value = map(New_Steering_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

						bitSet(_PCF_Data,7);
						bitClear(_PCF_Data,0);
						I2c.write(I2C_Add,0x02,_PCF_Data);

	 				}
	   				else{
	   	 				_Motor_2_PWM_Value=0;
	    				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

						bitClear(_PCF_Data,7);
						bitClear(_PCF_Data,0);
						I2c.write(I2C_Add,0x02,_PCF_Data);

	   				}
			}
	}

	_Drive_Val = New_Drive_Val;
	Drive_Motor();
}

void RCTractor::LiftingArms(int New_Rear_Val,int New_Front_Val){

	_Rear_Link_Val = map(New_Rear_Val, 0x00, 0xFF, _Rear_Link_Servo_Min, _Rear_Link_Servo_Max); // Scale data_val to between steering_min and steering_max
	_Rear_Link_Servo.write(_Rear_Link_Val);

	_Front_Link_Val = map(New_Front_Val, 0x00, 0xFF, _Front_Link_Servo_Min, _Front_Link_Servo_Max); // Scale data_val to between steering_min and steering_max
	_Front_Link_Servo.write(_Front_Link_Val);

}

void RCTractor::Secondary_Motor(int Second_Motor_Val){
	if(_Use_PCF8574==0){
					if (Second_Motor_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
			    		_Motor_2_PWM_Value = map(Second_Motor_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
	    				digitalWrite(_Motor_2_Direction_1_Pin, HIGH); // Set the direction with pins 7 and 8
	    				digitalWrite(_Motor_2_Direction_2_Pin, LOW);
					}
	   				else if (Second_Motor_Val > 135){
	    				_Motor_2_PWM_Value = map(Second_Motor_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
	   	 				digitalWrite(_Motor_2_Direction_2_Pin, HIGH); // Set the direction with pins 7 and 8
	    				digitalWrite(_Motor_2_Direction_1_Pin, LOW);
	 				}
	   				else{
	   	 				_Motor_2_PWM_Value=0;
	    				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
	    				digitalWrite(_Motor_2_Direction_1_Pin, LOW); // Set the direction with pins 7 and 8
	    				digitalWrite(_Motor_2_Direction_2_Pin, LOW);
	   				}
				}

				if(_Use_PCF8574==1){
					if (Second_Motor_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
			    		_Motor_2_PWM_Value = map(Second_Motor_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

						bitSet(_PCF_Data,0);
						bitClear(_PCF_Data,7);
						I2c.write(I2C_Add,0x02,_PCF_Data);

					}
	   				else if (Second_Motor_Val > 135){
	    				_Motor_2_PWM_Value = map(Second_Motor_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
						analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

						bitSet(_PCF_Data,7);
						bitClear(_PCF_Data,0);
						I2c.write(I2C_Add,0x02,_PCF_Data);

	 				}
	   				else{
	   	 				_Motor_2_PWM_Value=0;
	    				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

						bitClear(_PCF_Data,7);
						bitClear(_PCF_Data,0);
						I2c.write(I2C_Add,0x02,_PCF_Data);

	   				}
			}
}

void RCTractor::Drive_Motor(){
	if(_Motor_Config==1){
			if(_Use_PCF8574==0){
				if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		    		_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
    				digitalWrite(_Motor_1_Direction_1_Pin, HIGH); // Set the direction with pins 7 and 8
    				digitalWrite(_Motor_1_Direction_2_Pin, LOW);
				}
   				else if (_Drive_Val > 135){
    				_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
   	 				digitalWrite(_Motor_1_Direction_2_Pin, HIGH); // Set the direction with pins 7 and 8
    				digitalWrite(_Motor_1_Direction_1_Pin, LOW);
 				}
   				else{
   	 				_Motor_1_PWM_Value=0;
    				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
    				digitalWrite(_Motor_1_Direction_1_Pin, LOW); // Set the direction with pins 7 and 8
    				digitalWrite(_Motor_1_Direction_2_Pin, LOW);
   				}
			}

			if(_Use_PCF8574==1){
				if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		    		_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

					bitSet(_PCF_Data,4);
					bitClear(_PCF_Data,5);
					bitSet(_PCF_Data,3);
					I2c.write(I2C_Add,0x02,_PCF_Data);

				}
   				else if (_Drive_Val > 135){
    				_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
					analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

					bitSet(_PCF_Data,5);
					bitClear(_PCF_Data,4);
					bitClear(_PCF_Data,3);
					I2c.write(I2C_Add,0x02,_PCF_Data);

 				}
   				else{
   	 				_Motor_1_PWM_Value=0;
    				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

					bitClear(_PCF_Data,5);
					bitClear(_PCF_Data,4);
					bitClear(_PCF_Data,3);
					I2c.write(I2C_Add,0x02,_PCF_Data);

   				}
			}

	}

	if(_Motor_Config==2){
		if(_Use_PCF8574==0){

		}


		if(_Use_PCF8574==1){
			if(_Steering_Val<120 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
				_Diff_Adj_1 = map(_Steering_Val, 120, 0, 255,120);
				_Diff_Adj_2=255;
			}
			else if(_Steering_Val>135 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
			    _Diff_Adj_2 = map(_Steering_Val, 135, 255, 255,120);//drive_min, drive_max); // scale the data value for use with the motor
			    _Diff_Adj_1=255;
			}
			else{
				_Diff_Adj_1=255;
				_Diff_Adj_2=255;
			}

			if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
		    	_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
		    	_Motor_2_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor
				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

				if(_PCF_Data!=B11100000){
					bitSet(_PCF_Data,5);
					bitClear(_PCF_Data,4);
					bitSet(_PCF_Data,7);
					bitClear(_PCF_Data,0);
					bitSet(_PCF_Data,6);
					I2c.write(I2C_Add,0x02,_PCF_Data);
				}
			}
			else if (_Drive_Val > 135){
		    	_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
		    	_Motor_2_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor

				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor   // RCTractorDiff::write(0, 0);

				if(_PCF_Data!=B01010001){
					bitSet(_PCF_Data,4);
					bitClear(_PCF_Data,5);
					bitSet(_PCF_Data,0);
					bitClear(_PCF_Data,7);
					bitSet(_PCF_Data,6);
					I2c.write(I2C_Add,0x02,_PCF_Data);
				}
			}
			else{
				_Motor_1_PWM_Value=0;
		    	_Motor_2_PWM_Value=0;
				analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
				analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

				if(_PCF_Data!=B00000000){
					_PCF_Data = B00000000;
					I2c.write(I2C_Add,0x02,_PCF_Data);
				}
			}
		}
	}

	if(_Motor_Config==3){
			if(_Use_PCF8574==0){
				if(_Steering_Val<120 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
								_Diff_Adj_1 = map(_Steering_Val, 120, 0, 255,0);
							}
							else if(_Steering_Val>135 && _Motor_1_PWM_Pin!=0 && _Motor_2_PWM_Pin!=0){
							    _Diff_Adj_2 = map(_Steering_Val, 135, 255, 255,0);//drive_min, drive_max); // scale the data value for use with the motor
							}
							else{
								_Diff_Adj_1=255;
								_Diff_Adj_2=255;
							}

							if (_Drive_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
						    	_Motor_1_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
						    	_Motor_2_PWM_Value = map(_Drive_Val, 120, 0, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor
								analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
								analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

			    				digitalWrite(_Motor_1_Direction_1_Pin, HIGH);
			    				digitalWrite(_Motor_1_Direction_2_Pin, LOW);
			    				digitalWrite(_Motor_2_Direction_1_Pin, HIGH);
			    				digitalWrite(_Motor_2_Direction_2_Pin, LOW);
							}
							else if (_Drive_Val > 135){
						    	_Motor_1_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_1);//drive_min, drive_max); // scale the data value for use with the motor
						    	_Motor_2_PWM_Value = map(_Drive_Val, 135, 255, 0,_Diff_Adj_2);//drive_min, drive_max); // scale the data value for use with the motor

								analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
								analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor   // RCTractorDiff::write(0, 0);

			    				digitalWrite(_Motor_1_Direction_1_Pin, LOW);
			    				digitalWrite(_Motor_1_Direction_2_Pin, HIGH);
			    				digitalWrite(_Motor_2_Direction_1_Pin, LOW);
			    				digitalWrite(_Motor_2_Direction_2_Pin, HIGH);
							}
							else{
								_Motor_1_PWM_Value=0;
						    	_Motor_2_PWM_Value=0;
								analogWrite(_Motor_1_PWM_Pin,_Motor_1_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
								analogWrite(_Motor_2_PWM_Pin,_Motor_2_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

			    				digitalWrite(_Motor_1_Direction_1_Pin, LOW);
			    				digitalWrite(_Motor_1_Direction_2_Pin, LOW);
			    				digitalWrite(_Motor_2_Direction_1_Pin, LOW);
			    				digitalWrite(_Motor_2_Direction_2_Pin, LOW);
							}
			}

			if(_Use_PCF8574==1){

			}

	}
}

void RCTractor::Lights(int Light_Val){
	_Light_Val = Light_Val;

		if(_Old_Light_Val != _Light_Val){

			if((bitRead(_Light_Val,4)) == 1){ // Work Lights
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 255);} // Head Lights On
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, HIGH);} // Tail Lights On
			    if(_Work_Lights_Pin!=0){digitalWrite(_Work_Lights_Pin, HIGH);} // Work Lights On
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, HIGH);} // Accessory Lights On
			}
			else{
			    if(_Work_Lights_Pin!=0){digitalWrite(_Work_Lights_Pin, LOW);} // Work Lights Off
			}

			if((bitRead(_Light_Val,2)) == 1 && (bitRead(_Light_Val,3)) == 0){ // Dipped Head Lights
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 125);} // Head Lights 50%
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, HIGH);} // Tail Lights Off
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, HIGH);} // Accessory Lights On

			}

			if((bitRead(_Light_Val,2)) == 1 && (bitRead(_Light_Val,3)) == 1){  // Full Beams
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 255);} // Head Lights 100%
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, HIGH);} // Tail Lights On
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, HIGH);} // Accessory Lights On
			}

			if((bitRead(_Light_Val,2)) == 0 && (bitRead(_Light_Val,3)) == 0){ // All Off
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 0);} // Head Lights 0%
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, LOW);} // Tail Lights Off
				if(_Work_Lights_Pin!=0){digitalWrite(_Work_Lights_Pin, LOW);} // Work Lights Off
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, LOW);} // Accessory Lights Off
			}

			_Old_Light_Val = _Light_Val;
	}
}

void RCTractor::Timer(){
_Flash_Timer = _Flash_Timer+1;
	BeaconControl();

	if (_Flash_Timer == 1000 && _Flash_Timer2 == 10){ // If the desired on time is reached

		if((bitRead(_Light_Val,1)) == 1 && (bitRead(_Light_Val,0)) == 0){
	    	LeftIndicator();
	    }
	    else if((bitRead(_Light_Val,1)) == 0 && (bitRead(_Light_Val,0)) == 1){
	    	RightIndicator();
	    }
	    else if((bitRead(_Light_Val,1)) == 1 && (bitRead(_Light_Val,0)) == 1){
	    	HazardIndicator();
	    }
	    else{
	    	IndicatorsOff();
	    }
	    	_Flash_Timer = 0;
	    	_Flash_Timer2 = 0;
	}
	else if (_Flash_Timer == 1000){ // If the desired on time is reached
	  	_Flash_Timer = 0; // Clear ind_timer
	  	_Flash_Timer2 = _Flash_Timer2+1; // Incrememt ind_timer2
  	}
}

// Indicator control functions
void RCTractor::LeftIndicator(){
	if (_Left_Ind_Status == 0){ // If left indicator is off
    	if(_Left_Indicator_Pin!=0){digitalWrite(_Left_Indicator_Pin, HIGH);} // turn it on
    	_Left_Ind_Status = 1;
    	Serial.write(SOP);
    	Serial.write(0x02);
    	Serial.write(0x01);
    	Serial.write(EOP);

  	}
  	else{ // If left indicator is on
    	IndicatorsOff();
  	}
}

void RCTractor::RightIndicator(){
  	if (_Right_Ind_Status == 0){ // If right indicator is off
    	if(_Right_Indicator_Pin!=0){digitalWrite(_Right_Indicator_Pin, HIGH);} // turn it on
    	_Right_Ind_Status = 1;
    	Serial.write(SOP);
    	Serial.write(0x02);
    	Serial.write(0x02);
    	Serial.write(EOP);
  	}
  	else{ // If right indicator is on
    	IndicatorsOff();
  	}
}

void RCTractor::HazardIndicator(){
	if (_Left_Ind_Status == 0){ // If hazards are off
    	if(_Left_Indicator_Pin!=0){digitalWrite(_Left_Indicator_Pin, HIGH);} // turn them on
    	if(_Right_Indicator_Pin!=0){digitalWrite(_Right_Indicator_Pin, HIGH);}
    	Serial.write(SOP);
    	Serial.write(0x02);
    	Serial.write(0x03);
    	Serial.write(EOP);
    	_Left_Ind_Status = 1;
  	}
  	else{ // If hazards are on
    	IndicatorsOff();
    }
}

void RCTractor::IndicatorsOff(){
    if(_Left_Indicator_Pin!=0){digitalWrite(_Left_Indicator_Pin, LOW);} // turn them off
    if(_Right_Indicator_Pin!=0){digitalWrite(_Right_Indicator_Pin, LOW);}
    _Left_Ind_Status = 0;
    _Right_Ind_Status = 0;
	Serial.write(SOP);
	Serial.write(0x02);
	Serial.write(0x00);
	Serial.write(EOP);
}

void RCTractor::BeaconControl(){
if(_Beacon_Option==1){
    	if (_Flash_Timer2 == 0){
      		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
      		if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, HIGH);}
    	}
    	else if (_Flash_Timer2 == 5){
      		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, HIGH);}
      		if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
    	}
  	}
  	else if(_Beacon_Option==2){
    	if (_Flash_Timer2 == 0){
      		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 1){
    	  	if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
    	}
    	if (_Flash_Timer2 == 2){
    	  	if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 3){
    	  	if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
    	}

    	if (_Flash_Timer2 == 6){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 7){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
    	}
    	if (_Flash_Timer2 == 8){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 9){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
    	}
 	}
  	else{
  		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
    	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
  	}

}

long RCTractor::ReadVcc(){
	long result;
	// Read 1.1V reference against AVcc
	ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
	delay(2); // Wait for Vref to settle
	ADCSRA |= _BV(ADSC); // Convert
	while (bit_is_set(ADCSRA,ADSC));
	result = ADCL;
	result |= ADCH<<8;
	result = (1126400L / result); // Back-calculate AVcc in mV
	return result;
}
