/*
  RCTractor.h - Library for controlling RC Tractors.
  Created by Oisin O'Conchubhair, September 20, 2015.
  Released into the public domain.

  This library uses parts from pcf8574.cpp written by Rob Tillaart.
*/

#include "Arduino.h"
#include "RCTractorGuy_Excavator.h"
#include <RCTractorGuyI2C.h>
#include <Servo.h>
//#include "SoftPWM.h"

// Pin Assignments
static int _Steering_Servo_Pin = 0;
static int _Rear_Link_Servo_Pin = 0;
static int _Front_Link_Servo_Pin = 0;
static int _Motor_1_PWM_Pin = 0;
static int _Motor_1_Direction_1_Pin = 0;
static int _Motor_1_Direction_2_Pin = 0;
static int _Motor_2_PWM_Pin = 0;
static int _Motor_2_Direction_1_Pin = 0;
static int _Motor_2_Direction_2_Pin = 0;
static int _Head_Lights_Pin = 0;
static int _Tail_Lights_Pin = 0;
static int _Work_Lights_Pin = 0;
static int _Accessory_Lights_Pin = 0;
static int _Left_Indicator_Pin = 0;
static int _Right_Indicator_Pin = 0;
static int _Left_Beacon_Pin = 0;
static int _Right_Beacon_Pin = 0;

#define SOP '<'
#define EOP '>'

static int _Bucket_Mode;
static int _Extend_Mode;
static int _Raise_Mode;
static int _Slew_Mode;
static int _Track_Mode;
static int _Bucket_Control;
static int _Extend_Control;
static int _Raise_Control;
static int _Slew_Control;
static int _Left_Track_Control;
static int _Right_Track_Control;
static int _Bucket_Dir1;
static int _Extend_Dir1;
static int _Raise_Dir1;
static int _Slew_Dir1;
static int _Left_Track_Dir1;
static int _Right_Track_Dir1;
static int _Bucket_Dir2;
static int _Extend_Dir2;
static int _Raise_Dir2;
static int _Slew_Dir2;
static int _Left_Track_Dir2;
static int _Right_Track_Dir2;
static int _Bucket_Feedback;
static int _Extend_Feedback;
static int _Raise_Feedback;

static int _PCF_1_Data;
static int _PCF_2_Data;
static int _Bucket_PWM_Value;
static int _Extend_PWM_Value;
static int _Raise_PWM_Value;
static int _Slew_PWM_Value;
static int _Left_Track_PWM_Value;
static int _Right_Track_PWM_Value;

static int _Timer1_Counter;
static int _Light_Val;
static int _Old_Light_Val;
static int _Head_Lights;
static int _Brake_Lights;
static int _Spot_Lights;
static int _Feature_Lights;
static int _Flash_Timer;
static int _Flash_Timer2;
static int _Left_Ind_Status;
static int _Right_Ind_Status;
static int _Beacon_Option = 2;
static int _PCF8574 = 1;
static int _Drive_Val = 0;

// Motor Configuration Variables

static int _Motor_Config = 1;
static int _Use_PCF8574 = 0;

// Servo Variables
// ------------ Bucket Int setup ------------
static int _Bucket_Pos = 100;
static unsigned long _Bucket_Timer = millis();
static int _Bucket_Space = 1000;
static int _Bucket_Dir = 0;
static int _Bucket_Servo_Min = 0;
static int _Bucket_Servo_Max = 180;
static int _Bucket_Val = 0;

// ------------ Extend Int setup ------------
static int _Extend_Pos = 127;
static unsigned long _Extend_Timer = millis();
static int _Extend_Space = 1000;
static int _Extend_Dir = 0;
static int _Extend_Servo_Min = 0;
static int _Extend_Servo_Max = 180;
static int _Extend_Val = 0;

// ------------ Raise Int setup ------------
static int _Raise_Pos = 127;
static unsigned long _Raise_Timer = millis();
static int _Raise_Space = 1000;
static int _Raise_Dir = 0;
static int _Raise_Servo_Min = 0;
static int _Raise_Servo_Max = 180;
static int _Raise_Val = 0;


static int _Slew_Servo_Min = 0;
static int _Slew_Servo_Max = 180;
static int _Track1_Servo_Min = 0;
static int _Track1_Servo_Max = 180;
static int _Track2_Servo_Min = 0;
static int _Track2_Servo_Max = 180;

static int _Slew_Val = 0;
static int _Track1_Val = 0;
static int _Track2_Val = 0;

/************************************************************/
/*       Soft PWM Functions                                 */
/************************************************************/

RCExcavator::RCExcavator()
{
/*
	_Motor_1_PWM_Pin = Motor_1_PWM;
	_Motor_1_Direction_1_Pin = Motor_1_Dir_1;
	_Motor_1_Direction_2_Pin = Motor_1_Dir_2;
	_Motor_2_PWM_Pin = Motor_2_PWM;
	_Motor_2_Direction_1_Pin = Motor_2_Dir_1;
	_Motor_2_Direction_2_Pin = Motor_2_Dir_2;
	_Steering_Servo_Pin = Steering;
	_Rear_Link_Servo_Pin = Rear_Link;
	_Front_Link_Servo_Pin = Front_Link;
	_Left_Indicator_Pin = Left_Ind;
	_Right_Indicator_Pin = Right_Ind;
	_Head_Lights_Pin = Head_Lights;
	_Tail_Lights_Pin = Tail_Lights;
	_Work_Lights_Pin = Work_Lights;
	_Accessory_Lights_Pin = Accessory_Lights;
	_Left_Beacon_Pin = Left_Beacon;
	_Right_Beacon_Pin = Right_Beacon;


	if(_Motor_1_PWM_Pin!=0){pinMode(_Motor_1_PWM_Pin, OUTPUT);}
	if(_Motor_1_Direction_1_Pin!=0){pinMode(_Motor_1_Direction_1_Pin, OUTPUT);}
	if(_Motor_1_Direction_2_Pin!=0){pinMode(_Motor_1_Direction_2_Pin, OUTPUT);}
	if(_Motor_2_PWM_Pin!=0){pinMode(_Motor_2_PWM_Pin, OUTPUT);}
	if(_Motor_2_Direction_1_Pin!=0){pinMode(_Motor_2_Direction_1_Pin, OUTPUT);}
	if(_Motor_2_Direction_2_Pin!=0){pinMode(_Motor_2_Direction_2_Pin, OUTPUT);}
	if(_Steering_Servo_Pin!=0){pinMode(_Steering_Servo_Pin, OUTPUT);}
	if(_Rear_Link_Servo_Pin!=0){pinMode(_Rear_Link_Servo_Pin, OUTPUT);}
	if(_Front_Link_Servo_Pin!=0){pinMode(_Front_Link_Servo_Pin, OUTPUT);}
	if(_Left_Indicator_Pin!=0){pinMode(_Left_Indicator_Pin, OUTPUT);}
	if(_Right_Indicator_Pin!=0){pinMode(_Right_Indicator_Pin, OUTPUT);}
	if(_Head_Lights_Pin!=0){pinMode(_Head_Lights_Pin, OUTPUT);}
	if(_Tail_Lights_Pin!=0){pinMode(_Tail_Lights_Pin, OUTPUT);}
	if(_Work_Lights_Pin!=0){pinMode(_Work_Lights_Pin, OUTPUT);}
	if(_Accessory_Lights_Pin!=0){pinMode(_Accessory_Lights_Pin, OUTPUT);}
	if(_Left_Beacon_Pin!=0){pinMode(_Left_Beacon_Pin, OUTPUT);}
	if(_Right_Beacon_Pin!=0){pinMode(_Right_Beacon_Pin, OUTPUT);}

*/
	_PCF_1_Data = B00000000;
	_PCF_2_Data = B00000000;

	I2c.begin();
	I2c.write(0x38,0x02,_PCF_1_Data);
	I2c.write(0x3C,0x02,_PCF_2_Data);

	pinMode(2, OUTPUT);
	pinMode(3, OUTPUT);
	pinMode(4, OUTPUT);
	pinMode(7, OUTPUT);

	//SoftPWMBegin ();
	//SoftPWMSet (2, 0 );

}

void RCExcavator::Sleep(int wake){
	if(wake == 1){
	bitSet(_PCF_2_Data,5);
		I2c.write(0x3C,0x02,_PCF_2_Data);

}
else if(wake ==0){
	bitClear(_PCF_2_Data,5);
	I2c.write(0x3C,0x02,_PCF_2_Data);
}
}


    void RCExcavator::Bucket_Setup(int Mode, int Bucket_PWM, int Bucket_Dir_1, int Bucket_Dir_2, int Bucket_Feedback){
		_Bucket_Mode = Mode;

		if(Mode == 1){
			_Bucket_Control = 2;
			_Bucket_Feedback = Bucket_Feedback;
		}
		else if(Mode == 2){
			_Bucket_Servo.attach(Bucket_PWM);
			_Bucket_Servo_Min = Bucket_Dir_1;
			_Bucket_Servo_Max = Bucket_Dir_2;
			_Bucket_Pos = _Bucket_Servo_Min;
			_Bucket_Servo.write(_Bucket_Pos);
		}
		else if(Mode == 3){
		}


		}
    void RCExcavator::Extend_Setup(int Mode, int Extend_PWM, int Extend_Dir_1, int Extend_Dir_2, int Extend_Feedback){
		_Extend_Mode = Mode;

		if(Mode == 1){
			_Extend_Control = 3;
			_Extend_Feedback = Extend_Feedback;
		}
		else if(Mode == 2){
			_Extend_Servo.attach(Extend_PWM);
			_Extend_Servo_Min = Extend_Dir_1;
			_Extend_Servo_Max = Extend_Dir_2;
			_Extend_Pos = _Extend_Servo_Min;
			_Extend_Servo.write(_Extend_Pos);
		}
		else if(Mode == 3){
		}
		}
    void RCExcavator::Raise_Setup(int Mode, int Raise_PWM, int Raise_Dir_1, int Raise_Dir_2, int Raise_Feedback){
		_Raise_Mode = Mode;

		if(Mode == 1){
			_Raise_Control = 4;
			_Raise_Feedback = Raise_Feedback;
		}
		else if(Mode == 2){
			_Raise_Servo.attach(Raise_PWM);
			_Raise_Servo_Min = Raise_Dir_1;
			_Raise_Servo_Max = Raise_Dir_2;
			_Raise_Pos = _Raise_Servo_Min;
			_Raise_Servo.write(_Raise_Pos);
		}
		else if(Mode == 3){
		}
		}
    void RCExcavator::Slew_Setup(int Mode, int Slew_PWM, int Slew_Dir_1, int Slew_Dir_2){
		_Slew_Mode = Mode;

		if(Mode == 1){
			_Slew_Control = 5;
		}
		else if(Mode == 2){
			_Slew_Servo.attach(Slew_PWM);
			_Slew_Servo_Min = Slew_Dir_1;
			_Slew_Servo_Max = Slew_Dir_2;

		}
		else if(Mode == 3){
		}
		}
    void RCExcavator::Track_Setup(int Mode, int Left_Track_PWM, int Left_Track_Dir_1, int Left_Track_Dir_2, int Right_Track_PWM, int Right_Track_Dir_1, int Right_Track_Dir_2){
		_Track_Mode = Mode;

		if(Mode == 1){
			_Left_Track_Control = 6;
			_Right_Track_Control = 7;
		}
		else if(Mode == 2){
			_Track1_Servo.attach(Left_Track_PWM);
			_Track2_Servo.attach(Right_Track_PWM);
			_Track1_Servo_Min = Left_Track_Dir_1;
			_Track1_Servo_Max = Left_Track_Dir_2;
			_Track2_Servo_Min = Right_Track_Dir_1;
			_Track2_Servo_Max = Right_Track_Dir_2;
		}
		else if(Mode == 3){
		}
		}
    void RCExcavator::Control(int Bucket_Val,int Extend_Val,int Raise_Val,int Slew_Val,int Left_Track_Val, int Right_Track_Val){
			_Bucket_Val = Bucket_Val;
			_Extend_Val = Extend_Val;
			_Raise_Val = Raise_Val;
			_Slew_Val = Slew_Val;
			_Track1_Val = Left_Track_Val;
			_Track2_Val = Right_Track_Val;

			if(_Bucket_Mode==1){
						if (Bucket_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
				    		_Bucket_PWM_Value = map(Bucket_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
							//SoftPWMSet (2, _Bucket_PWM_Value);
					    	analogWrite(2,_Bucket_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

							bitClear(_PCF_2_Data,3);
							bitSet(_PCF_2_Data,4);
							I2c.write(0x3C,0x02,_PCF_2_Data);

						}
		   				else if (Bucket_Val > 135){
		    				_Bucket_PWM_Value = map(Bucket_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
							//SoftPWMSet (2, _Bucket_PWM_Value);; // Set the PWM vlue on pin 5 to move the motor
					    	analogWrite(2,_Bucket_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

							bitSet(_PCF_2_Data,3);
							bitClear(_PCF_2_Data,4);
							I2c.write(0x3C,0x02,_PCF_2_Data);
		 				}
		   				else{
		   	 				_Bucket_PWM_Value=0;
		    				//SoftPWMSet (2, _Bucket_PWM_Value); // Set the PWM vlue on pin 5 to move the motor
					    	analogWrite(2,_Bucket_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

							bitClear(_PCF_2_Data,3);
							bitClear(_PCF_2_Data,4);
							I2c.write(0x3C,0x02,_PCF_2_Data);
		   				}
			}


			if(_Extend_Mode==1){
						if (Extend_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
				    		_Extend_PWM_Value = map(Extend_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
							//pwm.duty[3] = _Extend_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    	analogWrite(3,_Extend_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

							bitSet(_PCF_2_Data,6);
							bitClear(_PCF_2_Data,7);
							I2c.write(0x3C,0x02,_PCF_2_Data);

						}
		   				else if (Extend_Val > 135){
		    				_Extend_PWM_Value = map(Extend_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
							//pwm.duty[3] = _Extend_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    	analogWrite(3,_Extend_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

							bitClear(_PCF_2_Data,6);
							bitSet(_PCF_2_Data,7);
							I2c.write(0x3C,0x02,_PCF_2_Data);
		 				}
		   				else{
		   	 				_Extend_PWM_Value=0;
		    				//pwm.duty[3] = _Extend_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    	analogWrite(3,_Extend_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

							bitClear(_PCF_2_Data,6);
							bitClear(_PCF_2_Data,7);
							I2c.write(0x3C,0x02,_PCF_2_Data);
		   				}
			}

			if(_Raise_Mode==1){
									if (Raise_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
							    		_Raise_PWM_Value = map(Raise_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										//pwm.duty[4] = _Raise_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    				analogWrite(4,_Raise_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,0);
										bitSet(_PCF_1_Data,1);
										I2c.write(0x38,0x02,_PCF_1_Data);

									}
					   				else if (Raise_Val > 135){
					    				_Raise_PWM_Value = map(Raise_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										//pwm.duty[4] = _Raise_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    				analogWrite(4,_Raise_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitSet(_PCF_1_Data,0);
										bitClear(_PCF_1_Data,1);
										I2c.write(0x38,0x02,_PCF_1_Data);
					 				}
					   				else{
					   	 				_Raise_PWM_Value=0;
					    				//pwm.duty[4] = _Raise_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    				analogWrite(4,_Raise_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,0);
										bitClear(_PCF_1_Data,1);
										I2c.write(0x38,0x02,_PCF_1_Data);
					   				}
			}

			if(_Slew_Mode==1){
									if (Slew_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
							    		_Slew_PWM_Value = map(Slew_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										analogWrite(_Slew_Control,_Slew_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitSet(_PCF_1_Data,2);
										bitClear(_PCF_1_Data,3);
										I2c.write(0x38,0x02,_PCF_1_Data);

									}
					   				else if (Slew_Val > 135){
					    				_Slew_PWM_Value = map(Slew_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										analogWrite(_Slew_Control,_Slew_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,2);
										bitSet(_PCF_1_Data,3);
										I2c.write(0x38,0x02,_PCF_1_Data);
					 				}
					   				else{
					   	 				_Slew_PWM_Value=0;
					    				analogWrite(_Slew_Control,_Slew_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,2);
										bitClear(_PCF_1_Data,3);
										I2c.write(0x38,0x02,_PCF_1_Data);
					   				}
			}

			if(_Track_Mode==1){
									if (Left_Track_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
							    		_Left_Track_PWM_Value = map(Left_Track_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										analogWrite(_Left_Track_Control,_Left_Track_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitSet(_PCF_1_Data,4);
										bitClear(_PCF_1_Data,5);
										I2c.write(0x38,0x02,_PCF_1_Data);

									}
					   				else if (Left_Track_Val > 135){
					    				_Left_Track_PWM_Value = map(Left_Track_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										analogWrite(_Left_Track_Control,_Left_Track_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,4);
										bitSet(_PCF_1_Data,5);
										I2c.write(0x38,0x02,_PCF_1_Data);
					 				}
					   				else{
					   	 				_Left_Track_PWM_Value=0;
					    				analogWrite(_Left_Track_Control,_Left_Track_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,4);
										bitClear(_PCF_1_Data,5);
										I2c.write(0x38,0x02,_PCF_1_Data);
					   				}

					   				if (Right_Track_Val < 120){ // These if functions leave a buffer zone of 15 bits above and below the drive center value
										_Right_Track_PWM_Value = map(Right_Track_Val, 120, 0, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										//pwm.duty[7] = _Bucket_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    				analogWrite(7,_Right_Track_PWM_Value); // Set the PWM vlue on pin 5 to move the motor


										bitSet(_PCF_1_Data,6);
										bitClear(_PCF_1_Data,7);
										I2c.write(0x38,0x02,_PCF_1_Data);

									}
									else if (Right_Track_Val > 135){
										_Right_Track_PWM_Value = map(Right_Track_Val, 135, 255, 0,255);//drive_min, drive_max); // scale the data value for use with the motor
										//pwm.duty[7] = _Bucket_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    				analogWrite(7,_Right_Track_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,6);
										bitSet(_PCF_1_Data,7);
										I2c.write(0x38,0x02,_PCF_1_Data);
									}
									else{
										_Right_Track_PWM_Value=0;
										//pwm.duty[7] = _Bucket_PWM_Value; // Set the PWM vlue on pin 5 to move the motor
					    				analogWrite(7,_Right_Track_PWM_Value); // Set the PWM vlue on pin 5 to move the motor

										bitClear(_PCF_1_Data,6);
										bitClear(_PCF_1_Data,7);
										I2c.write(0x38,0x02,_PCF_1_Data);
					   				}
			}

}



void RCExcavator::Lights(int Light_Val){
	_Light_Val = Light_Val;

		if(_Old_Light_Val != _Light_Val){

			if((bitRead(_Light_Val,4)) == 1){ // Work Lights
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 255);} // Head Lights On
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, HIGH);} // Tail Lights On
			    if(_Work_Lights_Pin!=0){digitalWrite(_Work_Lights_Pin, HIGH);} // Work Lights On
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, HIGH);} // Accessory Lights On
			}
			else{
			    if(_Work_Lights_Pin!=0){digitalWrite(_Work_Lights_Pin, LOW);} // Work Lights Off
			}

			if((bitRead(_Light_Val,2)) == 1){ // Dipped Head Lights
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 125);} // Head Lights 50%
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, HIGH);} // Tail Lights Off
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, HIGH);} // Accessory Lights On

			}
			else if((bitRead(_Light_Val,3)) == 1){  // Full Beams
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 255);} // Head Lights 100%
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, HIGH);} // Tail Lights On
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, HIGH);} // Accessory Lights On
			}
			else{ // All Off
			    if(_Head_Lights_Pin!=0){analogWrite(_Head_Lights_Pin, 0);} // Head Lights 0%
			    if(_Tail_Lights_Pin!=0){digitalWrite(_Tail_Lights_Pin, LOW);} // Tail Lights Off
				if(_Work_Lights_Pin!=0){digitalWrite(_Work_Lights_Pin, LOW);} // Work Lights Off
			    if(_Accessory_Lights_Pin!=0){digitalWrite(_Accessory_Lights_Pin, LOW);} // Accessory Lights Off
			}

			_Old_Light_Val = _Light_Val;
	}
}

void RCExcavator::Timer(){
_Flash_Timer = _Flash_Timer+1;
	BeaconControl();
	Servo_Update();

	if (_Flash_Timer == 1000 && _Flash_Timer2 == 10){ // If the desired on time is reached
	    	_Flash_Timer = 0;
	    	_Flash_Timer2 = 0;
	}
	else if (_Flash_Timer == 1000){ // If the desired on time is reached
	  	_Flash_Timer = 0; // Clear ind_timer
	  	_Flash_Timer2 = _Flash_Timer2+1; // Incrememt ind_timer2
  	}
}

void RCExcavator::Servo_Update(){

				_Slew_Val;
				_Track1_Val;
			_Track2_Val;


	if(_Bucket_Mode==2){
		if(_Bucket_Val < 120){
			_Bucket_Space = map(_Bucket_Val,120,0,250,19);
			_Bucket_Dir = 1;
		}
		else if(_Bucket_Val>135){
			_Bucket_Space = map(_Bucket_Val,135,255,250,19);
			_Bucket_Dir = 2;
		}
		else{_Bucket_Dir = 0;_Bucket_Space=300;}
			if((millis()-_Bucket_Timer)>_Bucket_Space){
				if (_Bucket_Pos != _Bucket_Servo_Min && _Bucket_Dir == 1){
					_Bucket_Pos --;
				}
				else if (_Bucket_Pos != _Bucket_Servo_Max && _Bucket_Dir == 2){
					_Bucket_Pos ++;
   				}
				_Bucket_Servo.write(_Bucket_Pos);
				_Bucket_Timer = millis();
    	}
	}

	if(_Extend_Mode==2){
		if(_Extend_Val < 120){
			_Extend_Space = map(_Extend_Val,120,0,250,19);
			_Extend_Dir = 1;
		}
		else if(_Extend_Val>135){
			_Extend_Space = map(_Extend_Val,135,255,250,19);
			_Extend_Dir = 2;
		}
		else{_Extend_Dir = 0;_Extend_Space=300;}
			if((millis()-_Extend_Timer)>_Extend_Space){
				if (_Extend_Pos != _Extend_Servo_Min && _Extend_Dir == 1){
					_Extend_Pos --;
				}
				else if (_Extend_Pos != _Extend_Servo_Max && _Extend_Dir == 2){
					_Extend_Pos ++;
   				}
				_Extend_Servo.write(_Extend_Pos);
				_Extend_Timer = millis();
    	}
	}

	if(_Raise_Mode==2){
		if(_Raise_Val < 120){
			_Raise_Space = map(_Raise_Val,125,0,250,19);
			_Raise_Dir = 1;
		}
		else if(_Raise_Val>135){
			_Raise_Space = map(_Raise_Val,130,255,250,19);
			_Raise_Dir = 2;
		}
		else{_Raise_Dir = 0;_Raise_Space=300;}
			if((millis()-_Raise_Timer)>_Raise_Space){
				if (_Raise_Pos != _Raise_Servo_Min && _Raise_Dir == 1){
					_Raise_Pos --;
				}
				else if (_Raise_Pos != _Raise_Servo_Max && _Raise_Dir == 2){
					_Raise_Pos ++;
   				}
				_Raise_Servo.write(_Raise_Pos);
				_Raise_Timer = millis();
    	}
	}



}


void RCExcavator::BeaconControl(){
if(_Beacon_Option==1){
    	if (_Flash_Timer2 == 0){
      		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
      		if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, HIGH);}
    	}
    	else if (_Flash_Timer2 == 5){
      		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, HIGH);}
      		if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
    	}
  	}
  	else if(_Beacon_Option==2){
    	if (_Flash_Timer2 == 0){
      		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 1){
    	  	if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
    	}
    	if (_Flash_Timer2 == 2){
    	  	if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 3){
    	  	if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
    	}

    	if (_Flash_Timer2 == 6){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 7){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
    	}
    	if (_Flash_Timer2 == 8){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, HIGH);}
    	}
    	if (_Flash_Timer2 == 9){
    	  	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
    	}
 	}
  	else{
  		if(_Left_Beacon_Pin!=0){digitalWrite(_Left_Beacon_Pin, LOW);}
    	if(_Right_Beacon_Pin!=0){digitalWrite(_Right_Beacon_Pin, LOW);}
  	}

}

long RCExcavator::ReadVcc(){
	long result;
	// Read 1.1V reference against AVcc
	ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
	delay(2); // Wait for Vref to settle
	ADCSRA |= _BV(ADSC); // Convert
	while (bit_is_set(ADCSRA,ADSC));
	result = ADCL;
	result |= ADCH<<8;
	result = (1126400L / result); // Back-calculate AVcc in mV
	return result;
}
