/*
Generic RC Trailer and Accessory Code Version 1.0
 This code was written by Oisin O'Conchubhair for the website www.rctractors.net
 This is the generic code used to control homemade RC Tractors.
 The code is intended to be used with an Arduino Pro Mini
 Circuit diagram can be found ar www.rctractors.net
 Pin Assignment
 D0 Serial RX
 D1 Serial TX
 D2 Left Indicator
 D3 Head Lights
 D4 Right Indicator
 D5 Drive Motor Speed Control
 D6 Brake Lights
 D7 Drive Motor Direction Control 1
 D8 Drive Motor Direction Control 2
 D9 Three-Point Linkage Servo Signal
 D10 Steering Servo Signal
 D11 Accessory Servo Signal
 D12 Accessory Serial TX Signal
 D13 Spot Lights
 A0 Beacon 1
 A1 Beacon 2
 A2 Not assigned
 A3 Not assigned
 A4 Not assigned
 A5 Not assigned
 */

// This section describes the vehicle
byte servo1_min = 0x50; // Servo 1 Minimum
byte servo1_max = 0x80; // Servo 1 Maximum

#define SOP '<'
#define EOP '>'

bool started = false;
bool ended = false;

uint8_t buf[16]; // Array to store recieved data
uint8_t command[16]; // Array to store recieved data

byte index;

// Variables used in the program
byte Data_1 = 0x00; // Stores the first byte in the command string
byte Command_Val = 0x00; // Stores the command value
byte Data_Val = 0x00; // Stores the data value
byte Old_Data_Val = 0x00; // Stores the data value
byte servo_val = 0x00; // Stores the servo position value
int Door_Button;
int Door_Open = 0;
int Lock_Button;
int Lock_Open = 0;
int Data_Counter = 0;
int Load = 2;
int Pickup = 0;
int Old_Func_Val = 0;
int Func_Val = 0;


#include <Servo.h> 
 
Servo servo1;  // Pick up reel servo 
Servo servo2;  // Rear door servo 
Servo servo3;  // Second bale release servo 

void setup() 
{
  Serial.begin(9600);

  pinMode(2, OUTPUT); // Right indicator
  pinMode(4, OUTPUT); // Left indicator
  pinMode(6, OUTPUT); // Motor PWM

servo1.attach(9);  // attaches the servo on pin 8 to the servo object

  servo1.write(0x75);      // Start in drive position
 }

void loop()
{
  check_serial();

  if(command[0] == 0x01){
	  servo_val = map(command[7], 0x00, 0xFF, servo1_min, servo1_max);  // Scale data_val to between the servo limits
	  servo1.write(servo_val);      // Set the servo position according to the scaled value
	  
	if((bitRead(command[9],2)) == 1){
	  	digitalWrite(6, HIGH);
	  }
	  else{
	  	digitalWrite(6, LOW);
	  }
  }
  else if(command[0] == 0x02){
	  if(command[1] == 0x01){
	  	digitalWrite(2, HIGH);
		digitalWrite(4, LOW);
	  }
	  else if(command[1] == 0x02){
	  	digitalWrite(2, LOW);
		digitalWrite(4, HIGH);
	  }
	  else if(command[1] == 0x03){
	  	digitalWrite(2, HIGH);
		digitalWrite(4, HIGH);
	  }
	  else if(command[1] == 0x00){
	  	digitalWrite(2, LOW);
		digitalWrite(4, LOW);
	  }
  }
}

void check_serial(){  
	  if (Serial.available()) { // Check if serial data is available
// Read all serial data available, as fast as possible
while(Serial.available() > 0)
{
char inChar = Serial.read();
if(inChar == SOP)
{
  index = 0;
  buf[index] = '\0';
  started = true;
  ended = false;
}
else if(inChar == EOP)
{
	if(index == 11){
		for (int i = 0; i < 12; i++) {
		command[i]=buf[i];
		}
	}
	else if(index == 2){
		 	   	  	 		for (int i = 0; i < 3; i++) {
		 	   	  	 		command[i]=buf[i];
		 	   	  	 		}
		 	}
  ended = true;
  break;
}
else
{
 if(index < 15)
 {
   buf[index] = inChar;
   index++;
   buf[index] = '\0';
 }
}
}

// We are here either because all pending serial
// data has been read OR because an end of
// packet marker arrived. Which is it?
if(started && ended)
{
// The end of packet marker arrived. Process the packet

// Reset for the next packet
started = false;
ended = false;
index = 0;
buf[index] = '\0';
}
	  }
}
